#include <vector>
#include <string>
#include <cctype>
#include <algorithm>

#include "constants.h"

// constants:
// exchanges: SMART, IDEAL, IDEALPRO, GLOBEX
// security types: CASH, STK, FUT, OPT
// bar sizes: 1 secs, 5 secs, 15 secs, 30 secs, 1 min, 2 mins, 3 mins, 5 mins, 15 mins, 30 mins, 1 hour, 1 day, 1 week, 1 month, 3 months, 1 year
// what to show: TRADES, MIDPOINT, BID, ASK, BID_ASK, HISTORICAL_VOLATILITY, OPTION_IMPLIED_VOLATILITY, OPTION_VOLUME
// duration units: integer{SPACE}unit (S|D|W|M|Y).

// generic tick: for CASH are: 100(Option Volume),101(Option Open Interest),104(Historical Volatility),105(Close Opt Volume),106(Option Implied Volatility),107(Close Implied Volatility),165(Misc. Stats),225(Auction),232/221(Mark Price),233(RTVolume),236(inventory),258/47(Fundamentals),291(Close Implied Volatility)
// generic tick: for FUT are: 100(Option Volume),101(Option Open Interest),104(Historical Volatility),105(Close Opt Volume),106(Option Implied Volatility),107(Close Implied Volatility),165(Misc. Stats),225(Auction),232/221(Mark Price),233(RTVolume),236(inventory),258/47(Fundamentals),291(Close Implied Volatility)

using namespace std;

namespace ib {
  static vector<string> c_datatype;
  static vector<string> c_barsize;

  void init() {

    c_datatype.push_back("TRADES");
    c_datatype.push_back("MIDPOINT");
    c_datatype.push_back("BID");
    c_datatype.push_back("ASK");
    c_datatype.push_back("BID_ASK");
    c_datatype.push_back("HISTORICAL_VOLATILITY");
    c_datatype.push_back("OPTION_IMPLIED_VOLATILITY");
    c_datatype.push_back("OPTION_VOLUME");

    c_barsize.push_back("1 secs");
    c_barsize.push_back("5 secs");
    c_barsize.push_back("15 secs");
    c_barsize.push_back("30 secs");
    c_barsize.push_back("1 min");
    c_barsize.push_back("2 mins");
    c_barsize.push_back("5 mins");
    c_barsize.push_back("15 mins");
    c_barsize.push_back("30 mins");
    c_barsize.push_back("1 hour");
    c_barsize.push_back("1 day");
    c_barsize.push_back("1 week");
    c_barsize.push_back("1 month");
    c_barsize.push_back("3 months");
    c_barsize.push_back("1 year");
    c_barsize.push_back("3 minutes");    
  }

  const string& datatype(enum DataType t) {
    return(c_datatype[t]);
  }
  
  const string& barsize(enum BarSize s) {
    return(c_barsize[s-1]);
  }
  
  BarSize string_to_barsize(char *spec) {
    string userspec(spec);
    transform(userspec.begin(), userspec.end(), userspec.begin(), ::tolower);
    
    for(unsigned int i=0; i<c_barsize.size(); i++)
      if(c_barsize[i] == userspec)
	return static_cast<ib::BarSize>(i+1);
    return S_15MIN; // return a sane default.
  }
  
  DataType string_to_datatype(char *spec) {
    string userspec(spec);
    transform(userspec.begin(), userspec.end(), userspec.begin(), ::toupper);
    
    for(unsigned int i=0; i<c_datatype.size(); i++)
      if(c_datatype[i] == userspec)
	return static_cast<ib::DataType>(i);
    return MIDPOINT; // return a sane default.
  }
}
