#include <sstream>

#include "csvwriter.h"

using namespace std;

string
csvWriter::genFilename(const Contract& c, time_t t) {
  stringstream ss;
  struct tm tm;
  char buffer[1024];
  
  ss << c.localSymbol << "-";
  localtime_r(&t, &tm);
  snprintf(buffer, 1024, "%04d%02d%02d.csv", tm.tm_year, tm.tm_mon, tm.tm_mday);
  ss << buffer;
  return ss.str();
}

string
csvWriter::genFilename(const Contract& c, const string ibtime) {
  stringstream ss;
  string temp;

  temp = ibtime;
  if(temp.find(' ') != string::npos)
    temp = temp.substr(0, temp.find(' '));
  ss << temp << "-";

  temp = c.localSymbol;
  if(temp.find('.') != string::npos)
    temp = temp.erase(temp.find('.'), 1);
  ss << temp << ".csv";

  return ss.str();
}

void
csvWriter::addHeader(handle h, const string name) {
  handleMapIterator it;
  
  it = hMap.find(h);
  if(it == hMap.end())
    return;

  it->second->headers.push_back(name);
}

void
csvWriter::writeHeader(handle h) {
  handleMapIterator it;
  
  it = hMap.find(h);
  if(it == hMap.end())
    return;

  vector<string>& headers = it->second->headers;
  ofstream& os = it->second->os;
  for(unsigned int i=0; i<headers.size(); i++) {
    os << headers[i];
    if(i != headers.size() - 1)
      os << ",";
  }
  os << endl;
}

csvWriter::handle
csvWriter::create(const string filename, bool truncate) {
  struct csvWriter::fileState* fs = new csvWriter::fileState;
  
  fs->firstitem = true;
  fs->os.open(filename.c_str(), truncate?ios_base::trunc:ios_base::app);
  if(fs->os.good()) {
    hMap.insert(make_pair(nextHandle, fs));
  }
  return nextHandle++;
}

void
csvWriter::close(handle h) {
  handleMapIterator it;
  
  it = hMap.find(h);
  if(it == hMap.end())
    return;

  it->second->os.close();
  delete it->second;
  hMap.erase(it);
}

void 
csvWriter::writeData(handle h) {
  handleMapIterator it;

  it = hMap.find(h);
  if(it == hMap.end())
    return;

  it->second->os << endl;
  it->second->os.flush();
  it->second->firstitem = true;
}

void
csvWriter::addComment(handle h, const string s) {
  handleMapIterator it;

  it = hMap.find(h);
  if(it == hMap.end())
    return;

  ofstream& os = it->second->os;
  os << "# " << s << endl;
}
