
struct RealtimeContainer {  
RealtimeContainer() : id(0),
    bid(0), ask(0), bidsize(0), asksize(0), open(0), close(0), high(0), low(0), volume(0), epx(0), esize(0) {}
  TickerId id;
  csvWriter::handle csvhandle;

  // current quote
  double bid;
  double ask;
  int bidsize;
  int asksize;

  // historical
  double open;
  double close;
  double high;
  double low;
  int volume;

  // execution
  double epx;
  unsigned int esize;
};

enum RealtimeUpdate {
  RT_NONE,
  RT_TOUCH,
  RT_EXECUTION
};

struct StatusContainer {
  std::string timestamp;
  std::string message;
};

class IBFetch : public EWrapper {
 public:
  IBFetch(LoggerRP& logger);
  ~IBFetch();
  
  // EWrapper api
  void tickPrice(TickerId tickerId, TickType field, double price, int canAutoExecute);
  void tickSize(TickerId tickerId, TickType field, int size);
  void tickOptionComputation( TickerId tickerId, TickType tickType, double impliedVol, double delta,
			      double modelPrice, double pvDividend);
  void tickGeneric(TickerId tickerId, TickType tickType, double value);
  void tickString(TickerId tickerId, TickType tickType, const IBString& value);
  void tickEFP(TickerId tickerId, TickType tickType, double basisPoints, const IBString& formattedBasisPoints,
	       double totalDividends, int holdDays, const IBString& futureExpiry, double dividendImpact, double dividendsToExpiry);
  void orderStatus(OrderId orderId, const IBString &status, int filled,
		   int remaining, double avgFillPrice, int permId, int parentId,
		   double lastFillPrice, int clientId, const IBString& whyHeld);
  void openOrder(OrderId orderId, const Contract&, const Order&, const OrderState&);
  void openOrderEnd();
  void winError(const IBString &str, int lastError);
  void connectionClosed();
  void updateAccountValue(const IBString& key, const IBString& val,
			  const IBString& currency, const IBString& accountName);
  void updatePortfolio(const Contract& contract, int position,
		       double marketPrice, double marketValue, double averageCost,
		       double unrealizedPNL, double realizedPNL, const IBString& accountName);
  void updateAccountTime(const IBString& timeStamp);
  void accountDownloadEnd(const IBString& accountName);
  void nextValidId(OrderId orderId);
  void contractDetails(int reqId, const ContractDetails& contractDetails);
  void bondContractDetails(int reqId, const ContractDetails& contractDetails);
  void contractDetailsEnd(int reqId);
  void execDetails(int reqId, const Contract& contract, const Execution& execution);
  void execDetailsEnd(int reqId);
  void error(const int id, const int errorCode, const IBString errorString);
  void updateMktDepth(TickerId id, int position, int operation, int side,
		      double price, int size);
  void updateMktDepthL2(TickerId id, int position, IBString marketMaker, int operation,
			int side, double price, int size);
  void updateNewsBulletin(int msgId, int msgType, const IBString& newsMessage, const IBString& originExch);
  void managedAccounts(const IBString& accountsList);
  void receiveFA(faDataType pFaDataType, const IBString& cxml);
  void historicalData(TickerId reqId, const IBString& date, double open, double high,
		      double low, double close, int volume, int barCount, double WAP, int hasGaps);
  void scannerParameters(const IBString &xml);
  void scannerData(int reqId, int rank, const ContractDetails &contractDetails,
		   const IBString &distance, const IBString &benchmark, const IBString &projection,
		   const IBString &legsStr);
  void scannerDataEnd(int reqId);
  void realtimeBar(TickerId reqId, long time, double open, double high, double low, double close,
		   long volume, double wap, int count);
  void currentTime(long time);
  void fundamentalData(TickerId reqId, const IBString& data);
  void deltaNeutralValidation(int reqId, const UnderComp& underComp);
  void tickSnapshotEnd(int reqId);

  // local
 public:
  void set_clientid(int id);
  void connect(const std::string& connspec) throw (std::runtime_error);
  void disconnect();
  bool saveHistoricalData(const Contract& c, const std::string& starttime, int ndays, ib::BarSize bsize, ib::DataType datatype);
  int saveRealtimeData(const Contract& c, const std::string& starttime);
  bool resolveContract(std::string& spec, std::string& exchange, std::string& currency, std::string& date, Contract& c);
  bool recvContractDetails(const Contract& c);
  int processReplies(bool drain, int timeout) throw(std::runtime_error);
  int requestsOutstanding() { return (_num_requests - _completed_requests); }
  int numIssues() { return _failed_requests; }
 public:
  std::stringstream status_ss;
  void showStatus();

 private:  
  void request_new();
  void request_fail();
  void request_done();

 private:
  std::auto_ptr<EPosixClientSocket> _pclient;
  int _clientid;
  ContractDetails details;
  bool have_details;
  csvWriter csv;
  int nrows;
  int _num_requests;
  int _completed_requests;
  int _failed_requests;
  TickerId realtimeTicker;
  typedef std::map<TickerId, RealtimeContainer*> RealtimeMap;
  typedef std::map<TickerId, RealtimeContainer*>::iterator RealtimeMapIterator;
  RealtimeMap rtmap;
  void writeRealtimeLine(RealtimeUpdate reason, RealtimeContainer* rtc);
  unsigned long realtimeCounter;
  LoggerRP& _logger;
};

