
#include <stdio.h>
#include <stdarg.h>

#include <stdexcept>
#include <string>

#include <boost/shared_ptr.hpp>

#include <log4cxx/logger.h>
#include <log4cxx/fileappender.h>
#include <log4cxx/patternlayout.h>

#include "logger.h"

using namespace std;

Logger::Logger() {
  _type = SYNC;
  _level = DEBUG;
}

Logger::~Logger() {
}

void
Logger::shutdown() {
}

void
Logger::init(const string& name, const string& filename) throw (runtime_error) {
  _name = name;

  if(!filename.size())
    throw runtime_error("Need a log file name specified"); 
  
  _logger = log4cxx::Logger::getLogger(filename);
  log4cxx::LayoutPtr layout(new log4cxx::PatternLayout("\%d \%-5p \%m\%n"));
  log4cxx::FileAppenderPtr appender(new log4cxx::FileAppender(layout, filename, true));
  _logger->addAppender(appender);  
}

void
Logger::log(level_t level, const char *fmt, ...) {
  if(level < _level)
    return;
  va_list(args);
  va_start(args, fmt);
  va_log(fmt, args);
  va_end(args);
 
  switch(level) {
  case DEBUG:
    LOG4CXX_DEBUG(_logger, tmp_buf);
    break;
  case INFO:
    LOG4CXX_INFO(_logger, tmp_buf);
    break;
  case WARN:
    LOG4CXX_WARN(_logger, tmp_buf);
    break;
  case ERROR:
    LOG4CXX_ERROR(_logger, tmp_buf);
    break;
  case FATAL:
    LOG4CXX_FATAL(_logger, tmp_buf);
    break;
  }
}

void
Logger::set_max_size(int size_in_mb) {
}

void
Logger::sync() throw (runtime_error) {
}

void
Logger::va_log(const char* fmt, va_list list) {
  vsnprintf(tmp_buf, Logger::MAX_LINELEN, fmt, list);
}

#ifdef TEST_LOGGER
int
main() {
  Logger l1;
  l1.init("l1", "testlog1.log");
  l1.log(Logger::DEBUG, "Main", __LINE__, "log statement 1: %s", "test1");
  l1.log(Logger::DEBUG, "Main", __LINE__, "log statement 2: %d", 123123);
  l1.set_level(Logger::WARN);
  l1.log(Logger::DEBUG, "Main", __LINE__, "devnull");
  l1.set_level(Logger::DEBUG);
  l1.log(Logger::DEBUG, "Main", __LINE__, "nondevnull");
  l1.sync();  
}

#endif
