#ifndef _LOGGER_H
#define _LOGGER_H

#undef DEBUG

class Logger {
 public:
  enum type_t { SYNC, ASYNC };
  enum level_t { DEBUG, INFO, WARN, ERROR, FATAL };
  const static int MAX_LINELEN=512;

 public:
  Logger();
  ~Logger();
  void init(const std::string& name, const std::string& filename) throw (std::runtime_error);
  void log(level_t level, const char *fmt, ...) __attribute__ ((__format__ (__printf__, 3, 4)));
  void sync() throw (std::runtime_error);
  void shutdown();
  void set_max_size(int size_in_mb);
  level_t get_level() { return _level; }
  void set_level(level_t level) { _level = level; }
  const std::string& name() { return _name; }
 private:
  void timestamp();
  void va_log(const char *fmt, va_list list);

 private:
  std::string _name;
  type_t _type;
  level_t _level;
  log4cxx::LoggerPtr _logger;
  char tmp_buf[MAX_LINELEN];
};

typedef boost::shared_ptr<Logger> LoggerRP;

#endif // _LOGGER_H
