#include <sys/time.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <limits.h>

#include <stdexcept>

#include "util.h"

using namespace std;

ostream& 
operator<<(ostream&o, const Contract& c) {
  o << "contract: " << c.symbol << "(" << c.localSymbol << "), sectype: " << c.secType << ", exchange: " << c.exchange 
    << ", currency: " << c.currency;
  // not useful
  // << ", primaryExch: " << c.primaryExchange << ", right: " << c.right << ", mult: " << c.multiplier;
  return o;
}
 
ostream& 
operator<<(ostream& out, const Order& o) {
  out << "order " << o.orderId << ", " << o.action << ", quantity: " << o.totalQuantity << ", limitpx: " << o.lmtPrice 
      << ", tif: " << o.tif << ", orderref: " << o.orderRef;
  return out;
}

ostream&
operator<<(ostream& o, const ContractDetails& cd) {
  cout << "contractDetails marketName: " << cd.marketName << ", priceMagnifier: " << cd.priceMagnifier << ", longName: " << cd.longName;
  return o;
}

string
timet_to_ibtime(time_t t, bool round_to_day) {
  struct tm res;
  
  if(!localtime_r(&t, &res))
    return "";

  char buffer[1024];
  strftime(buffer, 1024, IBTIME_FORMAT, &res);
  return buffer;
}

time_t
ibtime_to_timet(const string& ibtime_) throw (runtime_error) {
  struct tm tm;
  time_t timet;
  string ibtime = ibtime_;

  if(ibtime.find(" ") == string::npos) {
    if(ibtime.size() != 8)
      throw runtime_error("Invalid short date format. Must be YYYYMMDD");
    ibtime.append(" 00:00:00");
  }

  if(strptime(ibtime.c_str(), IBTIME_FORMAT, &tm)) {
    timet = mktime(&tm);
    return timet;
  }

  return static_cast<time_t>(0);
}

bool
ibtime_valid(const string& time_string) {
  return ibtime_to_timet(time_string)!=0;
}

void
gettimestamp(string& out) {
  struct timeval tv;
  struct tm res;

  ::gettimeofday(&tv, NULL);
  ::localtime_r(&tv.tv_sec, &res);
  char buffer[1024];
  strftime(buffer, 1024, TIMESTAMP_FORMAT, &res);
  char buffer2[1024];
  snprintf(buffer2, 1024, "%s.%06lu", buffer, tv.tv_usec);
  out = string(buffer2);
}

void
parse_connspec(const string& connspec, string& host, int& port) throw(runtime_error) {
  size_t sep = connspec.find(":");
  if(sep != string::npos) {
    host = connspec.substr(0, sep);
    port = ::atoi(connspec.substr(sep+1).c_str());
  } else {
    host = connspec;
    port = 7496;
  }

  struct hostent* he = ::gethostbyname(host.c_str());
  if(he == NULL)
    throw(runtime_error("Error: Could not resolve " + host));
  in_addr* address = (in_addr *)he->h_addr;
  host.assign(::inet_ntoa(*address));
}

// Limitations:
// 1. cutoff day is static... this should really be number of trading days before
//    expiration. 
// 2. schedule is hardcoded for CME futures

void
resolve_futures_contract(const string& name, const string& date, string& out, int cutoff_mday) {
  static char months_table[]="FGHJKMNQUVXZ";
  
  out = name; // failsafe; if we find an error, always return the input specification

  size_t posn;
  if((posn = name.find("c")) == string::npos)
    return;

  if(name.size() != posn+1+1)
    return;

  struct tm tm;
  time_t date_timet;
  try {
    date_timet = ibtime_to_timet(date);
  } catch (runtime_error& e) {
    return;
  }

  ::localtime_r(&date_timet, &tm);
  out = name.substr(0, posn);
  int month_code = tm.tm_mon;
  int year_code = tm.tm_year % 10;
  int contracts_forward = 1 - ::atoi(name.substr(posn+1, 1).c_str());

  // specific to e-minis at the moment
  if((month_code==2||month_code==5||month_code==8||month_code==11) && tm.tm_mday > cutoff_mday)
    contracts_forward++;

  month_code = 2 + (3*(contracts_forward+(month_code/3)));
  if(month_code > 11) {
    month_code = 2;
    year_code = (year_code+1)%10;
  }

  char year_digit = '0' + (year_code);
  char month_digit = months_table[month_code];
  out.append(1, month_digit);
  out.append(1, year_digit);
}
